import httpx
import os
import re
from dotenv import load_dotenv
from mcp.server.fastmcp import FastMCP

load_dotenv()

base_url = os.getenv("OI_API_BASE", "") + "/api/v1/retrieval/process/web"
authorization_token = os.getenv("OI_API_TOKEN", "")

# Initialize FastMCP server
mcp = FastMCP(
    "open-webui-web-search",
    prompt="""
# Open WebUI Search MCP Server

This server provides tools for searching the web using Microsoft Open WebUI's API.
It allows you to search for web pages, news articles, and images.

## Available Tools

### 1. open_webui_web_search
Use this tool for general web searches. Best for finding information,
websites, articles, and general content.

Example: "What is the capital of France?" or
"recipe for chocolate chip cookies"

## Guidelines for Use

- Keep queries concise and specific for best results

## Output Format

All search results will be formatted as text with clear sections for each
result item, including:

- Web search: Title, URL, and Description

If the API key is missing or invalid, appropriate error messages will be
returned.
""",
)    

def clean_content(content):
    # Remove excessive blank lines
    cleaned_content = re.sub(r'\n\s*\n+', '\n\n', content.strip())
    # Replace multiple spaces with a single space
    cleaned_content = re.sub(r'[ \t]+', ' ', cleaned_content)
    return cleaned_content
@mcp.tool()
async def get_web_search_results_from_oi(query: str) -> str:
    """Performs a web search using the Open WebUI Web Search API for general information
    and websites.

    Args:
        query: Search query (required)
    """
    if not base_url or not authorization_token:
        return (
            "Error: OI_API_BASE and/or  not OI_API_TOKEN not set. Please set the "
            "environment variables."
        )
        
    headers = {
        'Content-Type': 'application/json',
        'Authorization': f'Bearer {authorization_token}'
    }

    # Step 1: Get web search results (list of URLs)
    search_payload = {
        "queries": [query]
    }
    
    try:
        async with httpx.AsyncClient() as client:
            try:
                search_response = await client.post(f"{base_url}/search", headers=headers, json=search_payload)
            except httpx.RequestError as e:
                return f"Network error occurred while making the search API call: {str(e)}"
            if search_response.status_code != 200:
                raise Exception(f"Search API call failed: {search_response.status_code} - {search_response.text}")

            search_data = search_response.json()

            filenames = search_data.get("filenames", [])
            if not filenames:
                return "No filenames found in the search response."
            if not isinstance(filenames, list):
                return "Error: 'filenames' is not a list in the search response."

            combined_response = ""

            # Step 2: Loop through URLs to get page content
            for filename in filenames:
                process_payload = {
                    "url": filename
                }

                try:
                    process_response = await client.post(base_url, headers=headers, json=process_payload)
                except httpx.RequestError as e:
                    print(f"Network error occurred while processing URL {filename}: {str(e)}")
                    continue

                if process_response.status_code != 200:
                    print(f"Failed to process URL {filename}: {process_response.status_code} - {process_response.text}")
                    continue

                process_data = process_response.json()

                def safe_get(data, keys, default="No content available"):
                    for key in keys:
                        data = data.get(key, {})
                        if not isinstance(data, dict):
                            return default
                    return data if data else default

                content = safe_get(process_data, ["file", "data", "content"])

                # Append to get combined response
                cleaned_content = clean_content(content)
                combined_response += f"Source: {filename}\n\nContent:\n{cleaned_content}\n\n"

        return combined_response

    except httpx.HTTPError as e:
        return f"Error communicating with Open WebUI Websearch API: {str(e)}"
    except Exception as e:
        return f"Unexpected error: {str(e)}\n\nPayload: {query}"

def main():
    """Main entry point for the script."""
    # Initialize and run the server
    mcp.run()
    
if __name__ == "__main__":
    main()
    